


from jk_hwriter import HWriter

from .HTMLElement import *




class HTML5HeadElement(HTMLElement):

	def __init__(self, proto, name):
		assert name == "head"
		super().__init__(proto, "head")
	#

	def __hasMetaTagWithCharset(self):
		for child in self.children:
			if isinstance(child, HTMLElement):
				if child.name == "meta":
					if "charset" in child.attributes:
						return True
		return False
	#

	def _serialize(self, w:HWriter):
		w.write(self._openingTagData())

		if self._proto.bHasClosingTag:
			w.incrementIndent()

			bRequireExtraCharsetTag = not self.__hasMetaTagWithCharset()
			if self.children or bRequireExtraCharsetTag:
				w.lineBreak()
				if bRequireExtraCharsetTag:
					w.writeLn("<meta charset=\"UTF-8\">")
				for child in self.children:
					if isinstance(child, (int, float, str)):
						w.write(htmlEscape(str(child)))
					else:
						child._serialize(w)
				w.lineBreak()

			w.decrementIndent()
			w.write(self._closingTagData())

		else:
			if len(self.children) > 0:
				raise Exception("HTML tag \"" + self.name + "\" is not allowed to have child elements!")

		w.lineBreak()
	#

#










from jk_hwriter import HWriter

from .htmlgeneral import *




class HTMLRawCSS(object):

	def __init__(self, textOrTextList):
		if isinstance(textOrTextList, str):
			self.texts = [ textOrTextList ]
		else:
			self.texts = list(textOrTextList)
	#

	def __call__(self, **attrs):
		self.texts.append("".join(attrs))
		return self
	#

	def __getitem__(self, textOrTexts):
		if hasattr(type(textOrTexts), "__iter__"):
			self.texts.extend(textOrTexts)
		else:
			self.texts.append(textOrTexts)
		return self
	#

	def _serialize(self, w:HWriter):
		if self.texts:
			w.lineBreak()
			w.writeLn("<style type=\"text/css\">")
			w.incrementIndent()
			for text in self.texts:
				w.writeLn(text)
			w.decrementIndent()
			w.writeLn("</style>")
	#

#






from .Color import *



class ColorSpectrum(list):

	def __init__(self, name, *colorOrListOfColors):
		super().__init__(self)

		self.name = name

		if isinstance(colorOrListOfColors, Color):
			self.append(colorOrListOfColors)
		else:
			for c in colorOrListOfColors:
				assert isinstance(c, Color)
				self.append(c)
	#

	def getColor(self, f:float, bRaiseExceptionOnInvalidValue:bool = False):
		if bRaiseExceptionOnInvalidValue:
			if (f < 0) or (f > 1):
				raise Exception("Invalid value specified: " + repr(f))
		else:
			if f < 0:
				f = 0
			elif f > 1:
				f = 1

		if len(self) == 0:
			return Color.BLACK
		if len(self) == 1:
			return self[0]
		if f == 0:
			return self[0]
		if f == 1:
			return self[-1]

		f = f * (len(self) - 1)
		i = int(f)
		f = f - i

		c1 = self[i]
		c2 = self[i + 1]
		r = f * c2.r + (1 - f) * c1.r
		g = f * c2.g + (1 - f) * c1.g
		b = f * c2.b + (1 - f) * c1.b

		return Color(r, g, b)
	#

	def reversed(self):
		return ColorSpectrum(self[::-1])
	#

#



ColorSpectrum.RED_GREEN_BLUE_YELLOW_PINK_CYAN = ColorSpectrum(
	"RED_GREEN_BLUE_YELLOW_PINK_CYAN",
	Color.RED,
	Color.GREEN,
	Color.BLUE,
	Color.YELLOW,
	Color.PINK,
	Color.CYAN
)

ColorSpectrum.GREEN_BLUE_RED = ColorSpectrum(
	"GREEN_BLUE_RED",
	Color.GREEN,
	Color.BLUE,
	Color.RED,
)

ColorSpectrum.RED_BLUE_GREEN = ColorSpectrum(
	"RED_BLUE_GREEN",
	Color.RED,
	Color.BLUE,
	Color.GREEN,
)

ColorSpectrum.RED_GREEN_BLUE = ColorSpectrum(
	"RED_GREEN_BLUE",
	Color.RED,
	Color.GREEN,
	Color.BLUE,
)

ColorSpectrum.BLACK_BLUE_GREEN_RED_LRED = ColorSpectrum(
	"BLACK_BLUE_GREEN_RED_LRED",
	Color.BLACK,
	Color.BLUE,
	Color.GREEN,
	Color.RED,
	Color.LIGHT_RED,
)

ColorSpectrum.BLACK_WHITE = ColorSpectrum(
	"BLACK_WHITE",
	Color.BLACK,
	Color.WHITE,
)

ColorSpectrum.BLACK_RED = ColorSpectrum(
	"BLACK_RED",
	Color.BLACK,
	Color.RED,
)

ColorSpectrum.BLACK_GREEN = ColorSpectrum(
	"BLACK_GREEN",
	Color.BLACK,
	Color.GREEN,
)

ColorSpectrum.BLACK_BLUE = ColorSpectrum(
	"BLACK_BLUE",
	Color.BLACK,
	Color.BLUE,
)

ColorSpectrum.BLACK_RED_WHITE = ColorSpectrum(
	"BLACK_RED_WHITE",
	Color.BLACK,
	Color.RED,
	Color.WHITE,
)

ColorSpectrum.BLACK_GREEN_WHITE = ColorSpectrum(
	"BLACK_GREEN_WHITE",
	Color.BLACK,
	Color.GREEN,
	Color.WHITE,
)

ColorSpectrum.BLACK_BLUE_WHITE = ColorSpectrum(
	"BLACK_BLUE_WHITE",
	Color.BLACK,
	Color.BLUE,
	Color.WHITE,
)

ColorSpectrum.RAINBOW = ColorSpectrum(
	"RAINBOW",
	Color.BLACK,
	Color.RED,
	Color.YELLOW,
	Color.GREEN,
	Color.CYAN,
	Color.BLUE,
	Color.DARK_PINK,
	Color.BLACK,
)

ColorSpectrum.RAINBOW_REVERSED = ColorSpectrum(
	"RAINBOW_REVERSED",
	Color.BLACK,
	Color.DARK_PINK,
	Color.BLUE,
	Color.CYAN,
	Color.GREEN,
	Color.YELLOW,
	Color.RED,
	Color.BLACK,
)

ColorSpectrum.ALL = [
	ColorSpectrum.RED_GREEN_BLUE_YELLOW_PINK_CYAN,
	ColorSpectrum.GREEN_BLUE_RED,
	ColorSpectrum.RED_BLUE_GREEN,
	ColorSpectrum.RED_GREEN_BLUE,
	ColorSpectrum.BLACK_BLUE_GREEN_RED_LRED,
	ColorSpectrum.RED_GREEN_BLUE,
	ColorSpectrum.BLACK_WHITE,
	ColorSpectrum.BLACK_RED,
	ColorSpectrum.BLACK_GREEN,
	ColorSpectrum.BLACK_BLUE,
	ColorSpectrum.BLACK_RED_WHITE,
	ColorSpectrum.BLACK_GREEN_WHITE,
	ColorSpectrum.BLACK_BLUE_WHITE,
	ColorSpectrum.RAINBOW,
	ColorSpectrum.RAINBOW_REVERSED,
]












class Color(object):

	def __init__(self, r:float, g:float, b:float):
		self.r = r
		self.g = g
		self.b = b
	#

	def __repr__(self):
		return "Color(%0.2f, %0.2f, %0.2f)" % (self.r, self.g, self.b)
	#

	def __str__(self):
		return "Color(%0.2f, %0.2f, %0.2f)" % (self.r, self.g, self.b)
	#

	def toHTML(self):
		r, g, b = self.toRGB()
		return "#%0.2x%0.2x%0.2x" % (r, g, b)
	#

	def toRGB(self):
		if not isinstance(self.g, (int, float)) or (self.r < 0) or (self.r > 1):
			raise Exception("Red value must be a valid float value in the range [0..1]! (Value specified: " + str(self.r) + ")")
		if not isinstance(self.g, (int, float)) or (self.g < 0) or (self.g > 1):
			raise Exception("Green value must be a valid float value in the range [0..1]! (Value specified: " + str(self.g) + ")")
		if not isinstance(self.g, (int, float)) or (self.b < 0) or (self.b > 1):
			raise Exception("Blue value must be a valid float value in the range [0..1]! (Value specified: " + str(self.b) + ")")

		return int(round(self.r * 255)), int(round(self.g * 255)), int(round(self.b * 255))
	#

	def toConsoleBackGround(self):
		r, g, b = self.toRGB()

		return "\033[48;2;" + str(r) + ";" + str(g) + ";" + str(b) + "m"
	#

	def toConsoleForeGround(self):
		r, g, b = self.toRGB()

		return "\033[38;2;" + str(r) + ";" + str(g) + ";" + str(b) + "m"
	#

	@staticmethod
	def mix(*args):
		n = len(args)
		if n <= 1:
			raise Exception("Two or more color values required!")
		r = 0
		g = 0
		b = 0
		for c in args:
			r += c.r
			g += c.g
			b += c.b
		return Color(r / n, g / n, b / n)
	#

#



Color.CONSOLE_RESET_FG = "\033[39m"
Color.CONSOLE_RESET_BG = "\033[49m"

Color.BLACK = Color(0, 0, 0)
Color.DARK_GRAY = Color(0.25, 0.25, 0.25)
Color.GRAY = Color(0.5, 0.5, 0.5)
Color.LIGHT_GRAY = Color(0.75, 0.75, 0.75)
Color.WHITE = Color(1, 1, 1)

Color.DARK_RED = Color(0.5, 0, 0)
Color.DARK_GREEN = Color(0, 0.5, 0)
Color.DARK_BLUE = Color(0, 0, 0.5)

Color.DARK_YELLOW = Color(0.5, 0.5, 0)
Color.DARK_CYAN = Color(0, 0.5, 0.5)
Color.DARK_PINK = Color(0.5, 0, 0.5)

Color.LIGHT_CYAN = Color(0.5, 1, 1)
Color.LIGHT_PINK = Color(1, 0.5, 1)
Color.LIGHT_YELLOW = Color(1, 1, 0.5)

Color.LIGHT_BLUE = Color(0.5, 0.5, 1)
Color.LIGHT_RED = Color(1, 0.5, 0.5)
Color.LIGHT_GREEN = Color(0.5, 1, 0.5)

Color.BLUE = Color(0, 0, 1)
Color.RED = Color(1, 0, 0)
Color.GREEN = Color(0, 1, 0)

Color.CYAN = Color(0, 1, 1)
Color.PINK = Color(1, 0, 1)
Color.YELLOW = Color(1, 1, 0)



Color.DARK_ORANGE = Color(0.5, 0.25, 0)
Color.ORANGE = Color(1, 0.5, 0)








#!/usr/bin/python3


from .Color import Color
from .ColorSpectrum import ColorSpectrum
from .CSSMap import CSSMap

from .HTMLElement import HTMLElement
from .HTMLRawText import HTMLRawText
from .HTML5RootElement import HTML5RootElement

from .HTML5Scope import HTML5Scope



__version__ = "0.2019.9.11"








from jk_hwriter import HWriter

from .htmlgeneral import *
from .CSSMap import CSSMap




class HTMLElement(object):

	def __init__(self, proto, name):
		self.name = name
		self.attributes = {}
		self.children = []
		self._proto = proto
	#

	def addAttributes(self, **attrs):
		for k, v in attrs.items():
			if k.startswith("_"):
				self.attributes[k[1:]] = v
			else:
				self.attributes[k] = v
		return self
	#

	def addContent(self, *args):
		self.children.extend(args)
		return self
	#

	"""
	def addChildren(self, *args):
		self.children.extend(args)
		return self
	#
	"""

	def __call__(self, **attrs):
		assert not self.attributes
		for k, v in attrs.items():
			if k.startswith("_"):
				self.attributes[k[1:]] = v
			else:
				self.attributes[k] = v
		return self
	#

	def __getitem__(self, childOrChildren):
		if hasattr(type(childOrChildren), "__iter__"):
			for c1 in childOrChildren:
				if hasattr(type(c1), "__iter__"):
					for c2 in c1:
						if hasattr(type(c2), "__iter__"):
							self.children.extend(c2)
						else:
							self.children.append(c2)
				else:
					self.children.append(c1)
		else:
			self.children.append(childOrChildren)
		return self
	#

	def _openingTagData(self):
		ret = [ "<", self.name ]
		self._attrsToStr(ret)
		ret.append(">")
		return ret
	#

	def _attrsToStr(self, ret:list):
		for k, v in self.attributes.items():
			if k == "style":
				if isinstance(v, str):
					v = v.strip()
					if v:
						ret.extend((" style=\"", v, "\""))
				elif isinstance(v, CSSMap):
					if v:
						ret.extend((" style=\"", str(v), "\""))
				else:
					raise Exception("Unexpected value specified for HTML tag attribute '" + k + "': type " + str(type(v)) + ", value " + repr(v))
			else:
				if isinstance(v, (int, float)):
					ret.extend((" ", k, "=\"", str(v), "\""))
				elif isinstance(v, str):
					ret.extend((" ", k, "=\"", htmlEscape(v), "\""))
				else:
					raise Exception("Unexpected value specified for HTML tag attribute '" + k + "': type " + str(type(v)) + ", value " + repr(v))
	#

	def _closingTagData(self):
		return [ "</", self.name, ">" ]
	#

	def __str__(self):
		w = HWriter()
		self._serialize(w)
		return str(w)
	#

	def _serialize(self, w:HWriter):
		if self._proto.bLineBreakOuter:
			w.lineBreak()
		w.write(self._openingTagData())

		if self._proto.bHasClosingTag:
			w.incrementIndent()
			if self._proto.bLineBreakInner:
				if self.children:
					w.lineBreak()
					for child in self.children:
						if isinstance(child, (int, float, str)):
							w.write(htmlEscape(str(child)))
						else:
							child._serialize(w)
					w.lineBreak()
			else:
				for child in self.children:
					if isinstance(child, (int, float, str)):
						w.write(htmlEscape(str(child)))
					else:
						child._serialize(w)

			w.decrementIndent()
			w.write(self._closingTagData())

		else:
			if len(self.children) > 0:
				raise Exception("HTML tag \"" + self.name + "\" is not allowed to have child elements!")

		if self._proto.bLineBreakOuter:
			w.lineBreak()
	#

#










from .htmlgeneral import *
from .HTMLRawCSS import HTMLRawCSS




class _HTMLRawCSSProto(object):

	def __init__(self, implClass=HTMLRawCSS):
		self.implClass = implClass
	#

	def __call__(self, *args, **attrs):
		return self.implClass(args)()
	#

	def __getitem__(self, children):
		return self.implClass(children)
	#

#












from jk_hwriter import HWriter

from .htmlgeneral import *




class HTMLRawText(object):

	def __init__(self, textOrTextList):
		if isinstance(textOrTextList, str):
			self.texts1 = [ textOrTextList ]
		else:
			self.texts1 = list(textOrTextList)
		self.texts2 = []
	#

	def __call__(self, **attrs):
		self.texts1.append("".join(attrs))
		return self
	#

	def __getitem__(self, textOrTexts):
		if hasattr(type(textOrTexts), "__iter__"):
			self.texts2.extend(textOrTexts)
		else:
			self.texts2.append(textOrTexts)
		return self
	#

	def _serialize(self, w:HWriter):
		if self.texts1:
			for text in self.texts1:
				w.lineBreak()
				w.write(text)
				w.lineBreak()
		for text in self.texts2:
			w.write(text)
	#

#






from jk_hwriter import HWriter

from .HTMLElement import *
from .HTML5HeadElement import HTML5HeadElement




class HTML5RootElement(HTMLElement):

	def __init__(self, proto, name):
		assert name == "html"
		super().__init__(proto, "html")
	#

	def _openingTagData(self):
		ret = [ "<!DOCTYPE html>\n<html" ]
		if "lang" not in self.attributes:
			ret.append(" lang=\"en\"")
		self._attrsToStr(ret)
		ret.append(">")
		return ret
	#

	def __hasHeadTag(self):
		for child in self.children:
			if isinstance(child, HTMLElement):
				if child.name == "head":
					return True
		return False
	#

	def _serialize(self, w:HWriter):
		w.write(self._openingTagData())

		if self._proto.bHasClosingTag:
			w.incrementIndent()

			if self.children:
				w.lineBreak()
				if not self.__hasHeadTag():
					head = HTML5HeadElement(self._proto, "head")
					head._serialize(w)
				for child in self.children:
					if isinstance(child, (int, float, str)):
						w.write(htmlEscape(str(child)))
					else:
						child._serialize(w)
				w.lineBreak()

			w.decrementIndent()
			w.write(self._closingTagData())

		else:
			if len(self.children) > 0:
				raise Exception("HTML tag \"" + self.name + "\" is not allowed to have child elements!")

		w.lineBreak()
	#

#











from .Color import Color




class CSSMap(object):

	def __init__(self, *args, **kwargs):
		for k, v in kwargs.items():
			setattr(self, k, v)
	#

	def __bool__(self):
		for k in dir(self):
			if k.startswith("__"):
				continue
			return True
		return False
	#

	def __str__(self):
		ret = []
		for k in dir(self):
			if k.startswith("__"):
				continue

			if len(ret) > 0:
				ret.append(" ")

			ret.append(k.replace("_", "-"))
			ret.append(":")

			v = getattr(self, k)
			if isinstance(v, Color):
				ret.append(v.toHTML())
			elif isinstance(v, (int, float)):
				ret.append(str(v))
			elif isinstance(v, str):
				ret.append(v)
			else:
				raise Exception("Unexpected value type specified for CSS attribute '" + k + "': type " + str(type(v)) + ", value " + repr(v))
			ret.append(";")
		return "".join(ret)
	#

#










HTML_TAG_TYPE_STRUCTURE = 1
HTML_TAG_TYPE_INLINE_CONTENT = 2
HTML_TAG_TYPE_INLINE_ALL = 3




_HTML_ESCAPE_TABLE = {
	"&": "&amp;",
	"\"": "&quot;",
	"'": "&apos;",
	">": "&gt;",
	"<": "&lt;",
}

def htmlEscape(text:str):
	return "".join(_HTML_ESCAPE_TABLE.get(c, c) for c in text)
#












from .htmlgeneral import *
from ._HTMLElementProto import _HTMLElementProto, HTMLElement
from ._HTMLCommentProto import _HTMLCommentProto, HTMLComment
from ._HTMLRawTextProto import _HTMLRawTextProto, HTMLRawText
from ._HTMLRawCSSProto import _HTMLRawCSSProto, HTMLRawCSS
from .HTML5RootElement import HTML5RootElement
from .HTML5HeadElement import HTML5HeadElement





class HTML5Scope(object):

	a = _HTMLElementProto("a", tagType=HTML_TAG_TYPE_INLINE_ALL)
	abbr = _HTMLElementProto("abbr", tagType=HTML_TAG_TYPE_INLINE_ALL)
	address = _HTMLElementProto("address", tagType=HTML_TAG_TYPE_STRUCTURE)
	area = _HTMLElementProto("area", bHasClosingTag=False, tagType=HTML_TAG_TYPE_INLINE_CONTENT)
	article = _HTMLElementProto("article", tagType=HTML_TAG_TYPE_STRUCTURE)
	audio = _HTMLElementProto("audio", tagType=HTML_TAG_TYPE_STRUCTURE)
	b = _HTMLElementProto("b", tagType=HTML_TAG_TYPE_INLINE_ALL)
	big = _HTMLElementProto("big", tagType=HTML_TAG_TYPE_INLINE_ALL)
	base = _HTMLElementProto("base", bHasClosingTag=False, tagType=HTML_TAG_TYPE_INLINE_ALL)
	# bdi = _HTMLElementProto("bdi", tagType=XXXXX)
	# bdo = _HTMLElementProto("bdo", tagType=XXXXX)
	blockquote = _HTMLElementProto("blockquote", tagType=HTML_TAG_TYPE_STRUCTURE)
	body = _HTMLElementProto("body", tagType=HTML_TAG_TYPE_STRUCTURE)
	br = _HTMLElementProto("br", bHasClosingTag=False, tagType=HTML_TAG_TYPE_INLINE_CONTENT)
	button = _HTMLElementProto("button", tagType=HTML_TAG_TYPE_INLINE_CONTENT)
	# canvas = _HTMLElementProto("canvas", tagType=XXXXX)
	caption = _HTMLElementProto("caption", tagType=HTML_TAG_TYPE_INLINE_CONTENT)
	cite = _HTMLElementProto("cite", tagType=HTML_TAG_TYPE_INLINE_ALL)
	code = _HTMLElementProto("code", tagType=HTML_TAG_TYPE_STRUCTURE)
	col = _HTMLElementProto("col", bHasClosingTag=False, tagType=HTML_TAG_TYPE_INLINE_CONTENT)
	colgroup = _HTMLElementProto("colgroup", tagType=HTML_TAG_TYPE_STRUCTURE)
	command = _HTMLElementProto("command", bHasClosingTag=False, tagType=HTML_TAG_TYPE_INLINE_ALL)
	data = _HTMLElementProto("data", tagType=HTML_TAG_TYPE_INLINE_ALL)
	datalist = _HTMLElementProto("datalist", tagType=HTML_TAG_TYPE_STRUCTURE)
	_del = _HTMLElementProto("del", tagType=HTML_TAG_TYPE_INLINE_ALL)			# problem
	details = _HTMLElementProto("details", tagType=HTML_TAG_TYPE_STRUCTURE)
	dfn = _HTMLElementProto("dfn", tagType=HTML_TAG_TYPE_INLINE_ALL)
	dl = _HTMLElementProto("dl", tagType=HTML_TAG_TYPE_STRUCTURE)
	dt = _HTMLElementProto("dt", tagType=HTML_TAG_TYPE_INLINE_CONTENT)
	dd = _HTMLElementProto("dd", tagType=HTML_TAG_TYPE_INLINE_CONTENT)
	div = _HTMLElementProto("div", tagType=HTML_TAG_TYPE_STRUCTURE)
	em = _HTMLElementProto("em", tagType=HTML_TAG_TYPE_INLINE_ALL)
	embed = _HTMLElementProto("embed", bHasClosingTag=False, tagType=HTML_TAG_TYPE_INLINE_CONTENT)
	fieldset = _HTMLElementProto("fieldset", tagType=HTML_TAG_TYPE_STRUCTURE)
	figcaption = _HTMLElementProto("figcaption", tagType=HTML_TAG_TYPE_INLINE_CONTENT)
	figure = _HTMLElementProto("figure", tagType=HTML_TAG_TYPE_STRUCTURE)
	footer = _HTMLElementProto("footer", tagType=HTML_TAG_TYPE_STRUCTURE)
	form = _HTMLElementProto("form", tagType=HTML_TAG_TYPE_STRUCTURE)
	h1 = _HTMLElementProto("h1", tagType=HTML_TAG_TYPE_INLINE_CONTENT)
	h2 = _HTMLElementProto("h2", tagType=HTML_TAG_TYPE_INLINE_CONTENT)
	h3 = _HTMLElementProto("h3", tagType=HTML_TAG_TYPE_INLINE_CONTENT)
	h4 = _HTMLElementProto("h4", tagType=HTML_TAG_TYPE_INLINE_CONTENT)
	h5 = _HTMLElementProto("h5", tagType=HTML_TAG_TYPE_INLINE_CONTENT)
	h6 = _HTMLElementProto("h6", tagType=HTML_TAG_TYPE_INLINE_CONTENT)
	head = _HTMLElementProto("head", tagType=HTML_TAG_TYPE_STRUCTURE, implClass=HTML5HeadElement)
	header = _HTMLElementProto("header", tagType=HTML_TAG_TYPE_STRUCTURE)
	hr = _HTMLElementProto("hr", bHasClosingTag=False, tagType=HTML_TAG_TYPE_STRUCTURE)
	html = _HTMLElementProto("html", tagType=HTML_TAG_TYPE_STRUCTURE, implClass=HTML5RootElement)
	i = _HTMLElementProto("i", tagType=HTML_TAG_TYPE_INLINE_ALL)
	img = _HTMLElementProto("img", bHasClosingTag=False, tagType=HTML_TAG_TYPE_INLINE_ALL)
	_input = _HTMLElementProto("input", bHasClosingTag=False, tagType=HTML_TAG_TYPE_INLINE_CONTENT)			# problem
	ins = _HTMLElementProto("ins", tagType=HTML_TAG_TYPE_INLINE_ALL)
	# kbd = _HTMLElementProto("kbd", tagType=XXXXX)
	# keygen = _HTMLElementProto("keygen", bHasClosingTag=False, tagType=XXXXX)
	label = _HTMLElementProto("label", bHasClosingTag=False, tagType=HTML_TAG_TYPE_INLINE_CONTENT)
	legend = _HTMLElementProto("legend", tagType=HTML_TAG_TYPE_STRUCTURE)
	li = _HTMLElementProto("li", tagType=HTML_TAG_TYPE_INLINE_CONTENT)
	link = _HTMLElementProto("link", bHasClosingTag=False, tagType=HTML_TAG_TYPE_INLINE_CONTENT)
	main = _HTMLElementProto("main", tagType=HTML_TAG_TYPE_STRUCTURE)
	_map = _HTMLElementProto("map", tagType=HTML_TAG_TYPE_STRUCTURE)			# problem
	mark = _HTMLElementProto("mark", tagType=HTML_TAG_TYPE_INLINE_ALL)
	# math = _HTMLElementProto("math", tagType=XXXXX)		# https://developer.mozilla.org/de/docs/Web/MathML/Element/math
	menu = _HTMLElementProto("menu", tagType=HTML_TAG_TYPE_STRUCTURE)
	menuitem = _HTMLElementProto("menuitem", tagType=HTML_TAG_TYPE_INLINE_CONTENT)
	meta = _HTMLElementProto("meta", bHasClosingTag=False, tagType=HTML_TAG_TYPE_INLINE_CONTENT)
	meter = _HTMLElementProto("meter", tagType=HTML_TAG_TYPE_INLINE_ALL)
	_object = _HTMLElementProto("object", tagType=HTML_TAG_TYPE_STRUCTURE)		# problem
	ol = _HTMLElementProto("ol", tagType=HTML_TAG_TYPE_STRUCTURE)
	optgroup = _HTMLElementProto("optgroup", tagType=HTML_TAG_TYPE_STRUCTURE)
	option = _HTMLElementProto("option", bHasClosingTag=False, tagType=HTML_TAG_TYPE_INLINE_CONTENT)
	output = _HTMLElementProto("output", tagType=HTML_TAG_TYPE_INLINE_CONTENT)
	p = _HTMLElementProto("p", tagType=HTML_TAG_TYPE_INLINE_CONTENT)
	param = _HTMLElementProto("param", bHasClosingTag=False, tagType=HTML_TAG_TYPE_INLINE_CONTENT)
	picture = _HTMLElementProto("picture", tagType=HTML_TAG_TYPE_STRUCTURE)
	# pre = _HTMLElementProto("pre", tagType=XXXXX)
	progress = _HTMLElementProto("progress", tagType=HTML_TAG_TYPE_INLINE_ALL)
	q = _HTMLElementProto("q", tagType=HTML_TAG_TYPE_STRUCTURE)
	section = _HTMLElementProto("section", tagType=HTML_TAG_TYPE_STRUCTURE)
	s = _HTMLElementProto("s", tagType=HTML_TAG_TYPE_INLINE_ALL)
	samp = _HTMLElementProto("samp", tagType=HTML_TAG_TYPE_INLINE_ALL)
	# script = _HTMLElementProto("script", tagType=XXXXX)
	select = _HTMLElementProto("select", tagType=HTML_TAG_TYPE_STRUCTURE)
	small = _HTMLElementProto("small", tagType=HTML_TAG_TYPE_INLINE_ALL)
	source = _HTMLElementProto("source", bHasClosingTag=False, tagType=HTML_TAG_TYPE_INLINE_CONTENT)
	span = _HTMLElementProto("span", tagType=HTML_TAG_TYPE_INLINE_ALL)
	strong = _HTMLElementProto("strong", tagType=HTML_TAG_TYPE_INLINE_ALL)
	# style = _HTMLElementProto("style", bHasClosingTag=False, tagType=XXXXX)
	summary = _HTMLElementProto("summary", tagType=HTML_TAG_TYPE_INLINE_CONTENT)
	sub = _HTMLElementProto("sub", tagType=HTML_TAG_TYPE_INLINE_ALL)
	sup = _HTMLElementProto("sup", tagType=HTML_TAG_TYPE_INLINE_ALL)
	# svg = _HTMLElementProto("svg", tagType=XXXXX)
	table = _HTMLElementProto("table", tagType=HTML_TAG_TYPE_STRUCTURE)
	textarea = _HTMLElementProto("textarea", tagType=HTML_TAG_TYPE_INLINE_CONTENT)
	tbody = _HTMLElementProto("tbody", tagType=HTML_TAG_TYPE_STRUCTURE)
	td = _HTMLElementProto("td", tagType=HTML_TAG_TYPE_STRUCTURE)
	tfoot = _HTMLElementProto("tfoot", tagType=HTML_TAG_TYPE_STRUCTURE)
	th = _HTMLElementProto("td", tagType=HTML_TAG_TYPE_STRUCTURE)
	thead = _HTMLElementProto("thead", tagType=HTML_TAG_TYPE_STRUCTURE)
	time = _HTMLElementProto("time", tagType=HTML_TAG_TYPE_INLINE_ALL)
	title = _HTMLElementProto("title", tagType=HTML_TAG_TYPE_INLINE_CONTENT)
	tr = _HTMLElementProto("tr", tagType=HTML_TAG_TYPE_STRUCTURE)
	track = _HTMLElementProto("track", bHasClosingTag=False, tagType=HTML_TAG_TYPE_INLINE_CONTENT)
	u = _HTMLElementProto("u", tagType=HTML_TAG_TYPE_INLINE_ALL)
	ul = _HTMLElementProto("ul", tagType=HTML_TAG_TYPE_STRUCTURE)
	var = _HTMLElementProto("var", tagType=HTML_TAG_TYPE_STRUCTURE)
	video = _HTMLElementProto("video", tagType=HTML_TAG_TYPE_STRUCTURE)
	wbr = _HTMLElementProto("wbr", bHasClosingTag=False, tagType=HTML_TAG_TYPE_INLINE_ALL)
	# TODO: add support for comments

	def __enter__(self):
		return self
	#

	def __exit__(self, exc_type, exc_val, exc_tb):
		pass
	#

	comment = _HTMLCommentProto()

	raw_html = _HTMLRawTextProto()

	raw_style_css = _HTMLRawCSSProto()

#













################################################################################
################################################################################
###
###  This file is automatically generated. Do not change this file! Changes
###  will get overwritten! Change the source file for "setup.py" instead.
###  This is either 'packageinfo.json' or 'packageinfo.jsonc'
###
################################################################################
################################################################################


from setuptools import setup

def readme():
	with open("README.md", "r", encoding="UTF-8-sig") as f:
		return f.read()

setup(
	author = "Jürgen Knauth",
	author_email = "pubsrc@binary-overflow.de",
	classifiers = [
		"Programming Language :: Python :: 3",
		"Development Status :: 4 - Beta",
		"License :: OSI Approved :: Apache Software License",
	],
	description = "This python module provides support for programmatically generating HTML5 code.",
	download_url = "https://github.com/jkpubsrc/python-module-jk-rawhtml/tarball/0.2019.9.11",
	include_package_data = False,
	install_requires = [
		"jk_hwriter",
	],
	keywords = [
		"html",
		"css",
		"html5",
	],
	license = "Apache 2.0",
	name = "jk_rawhtml",
	packages = [
		"jk_rawhtml",
	],
	url = "https://github.com/jkpubsrc/python-module-jk-rawhtml",
	version = "0.2019.9.11",
	zip_safe = False,
	long_description = readme(),
	long_description_content_type="text/markdown",
)





from .HTMLElement import HTMLElement
from .htmlgeneral import *




class _HTMLElementProto(object):

	def __init__(self, name:str, bHasClosingTag=True, tagType=HTML_TAG_TYPE_INLINE_CONTENT, implClass=HTMLElement, extraAttributes:dict=None):
		assert isinstance(name, str)
		assert isinstance(bHasClosingTag, bool)

		self.name = name
		self.bHasClosingTag = bHasClosingTag
		self.tagType = tagType
		self.implClass = implClass

		if tagType == HTML_TAG_TYPE_INLINE_CONTENT:
			self.bLineBreakOuter = True
			self.bLineBreakInner = False
		elif tagType == HTML_TAG_TYPE_INLINE_ALL:
			self.bLineBreakOuter = False
			self.bLineBreakInner = False
		elif tagType == HTML_TAG_TYPE_STRUCTURE:
			self.bLineBreakOuter = True
			self.bLineBreakInner = True
		else:
			raise Exception("Invalid tag type specified: " + str(tagType))

		if extraAttributes:
			assert isinstance(extraAttributes, dict)
			self.extraAttributes = extraAttributes
		else:
			self.extraAttributes = None
	#

	def __call__(self, *args, **attrs):
		if self.extraAttributes:
			d = dict(self.extraAttributes)
			d.update(attrs)
			return self.implClass(self, self.name)(**d)
		else:
			return self.implClass(self, self.name)(**attrs)
	#

	def __getitem__(self, children):
		return self.implClass(self, self.name)[children]
	#

#







